"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DEFAULT_MARK_SCORE_VOTE_THRESHOLD = void 0;
const ballot_encoder_1 = require("@votingworks/ballot-encoder");
const assert_1 = require("assert");
const debug_1 = __importDefault(require("debug"));
const jsfeat = __importStar(require("jsfeat"));
const util_1 = require("util");
const uuid_1 = require("uuid");
const getVotesFromMarks_1 = __importDefault(require("./getVotesFromMarks"));
const findContestOptions_1 = __importDefault(require("./hmpb/findContestOptions"));
const findContests_1 = __importStar(require("./hmpb/findContests"));
const findTargets_1 = __importDefault(require("./hmpb/findTargets"));
const metadata_1 = require("./metadata");
const binarize_1 = require("./utils/binarize");
const crop_1 = __importDefault(require("./utils/crop"));
const defined_1 = __importDefault(require("./utils/defined"));
const flip_1 = require("./utils/flip");
const geometry_1 = require("./utils/geometry");
const iterators_1 = require("./utils/iterators");
const diff_1 = __importStar(require("./utils/jsfeat/diff"));
const matToImageData_1 = __importDefault(require("./utils/jsfeat/matToImageData"));
const readGrayscaleImage_1 = __importDefault(require("./utils/jsfeat/readGrayscaleImage"));
const KeyedMap_1 = __importDefault(require("./utils/KeyedMap"));
const offsets_1 = __importDefault(require("./utils/offsets"));
const outline_1 = __importDefault(require("./utils/outline"));
const debug = debug_1.default('hmpb-interpreter:Interpreter');
exports.DEFAULT_MARK_SCORE_VOTE_THRESHOLD = 0.12;
/**
 * Interprets ballot images based on templates. A template is simply an empty
 * ballot and should be exactly what is given to a voter for them to mark.
 *
 * @example
 *
 * ```ts
 * const interpreter = new Interpreter(election)
 * await interpreter.addTemplate(templatePage1)
 * await interpreter.addTemplate(templatePage2)
 * console.log(await interpreter.interpretBallot(ballotPage1))
 * ```
 */
class Interpreter {
    constructor(optionsOrElection) {
        var _a, _b, _c, _d, _e, _f, _g;
        this.templates = new KeyedMap_1.default(([locales, ballotStyleId, precinctId, pageNumber]) => [
            locales === null || locales === void 0 ? void 0 : locales.primary,
            locales === null || locales === void 0 ? void 0 : locales.secondary,
            ballotStyleId,
            precinctId,
            pageNumber,
        ].join('-'));
        if ('election' in optionsOrElection) {
            this.election = optionsOrElection.election;
            this.detectQRCode = (_a = optionsOrElection.detectQRCode) !== null && _a !== void 0 ? _a : optionsOrElection.decodeQRCode;
            this.markScoreVoteThreshold = (_d = (_b = optionsOrElection.markScoreVoteThreshold) !== null && _b !== void 0 ? _b : (_c = this.election.markThresholds) === null || _c === void 0 ? void 0 : _c.definite) !== null && _d !== void 0 ? _d : exports.DEFAULT_MARK_SCORE_VOTE_THRESHOLD;
            this.testMode = (_e = optionsOrElection.testMode) !== null && _e !== void 0 ? _e : false;
        }
        else {
            this.election = optionsOrElection;
            this.markScoreVoteThreshold = (_g = (_f = this.election.markThresholds) === null || _f === void 0 ? void 0 : _f.definite) !== null && _g !== void 0 ? _g : exports.DEFAULT_MARK_SCORE_VOTE_THRESHOLD;
            this.testMode = false;
        }
    }
    async addTemplate(imageDataOrTemplate, metadata, { flipped } = {}) {
        const template = 'data' in imageDataOrTemplate
            ? await this.interpretTemplate(imageDataOrTemplate, metadata, {
                flipped,
            })
            : imageDataOrTemplate;
        if (!metadata) {
            metadata = template.ballotImage.metadata;
        }
        if (metadata.isTestMode !== this.testMode) {
            throw new Error(`interpreter configured with testMode=${this.testMode} cannot add templates with isTestMode=${metadata.isTestMode}`);
        }
        this.setTemplate(metadata, template);
        return template;
    }
    /**
     * Gets a template by ballot style, precinct, and page number if present.
     */
    getTemplate({ locales, ballotStyleId, precinctId, pageNumber, }) {
        return this.templates.get([locales, ballotStyleId, precinctId, pageNumber]);
    }
    /**
     * Sets a template by ballot style, precinct, and page number.
     */
    setTemplate({ locales, ballotStyleId, precinctId, pageNumber }, template) {
        this.templates.set([locales, ballotStyleId, precinctId, pageNumber], template);
    }
    /**
     * Interprets an image as a template, returning the layout information read
     * from the image. The template image should be an image of a blank ballot,
     * either scanned or otherwise rendered as an image.
     */
    async interpretTemplate(imageData, metadata, { flipped = false } = {}) {
        debug('interpretTemplate: looking for contests in %d×%d image', imageData.width, imageData.height);
        ({ imageData, metadata } = await this.normalizeImageDataAndMetadata(imageData, metadata, {
            flipped,
        }));
        debug('using metadata for template: %O', metadata);
        const contests = findContestOptions_1.default([
            ...iterators_1.map(this.findContests(imageData).contests, ({ bounds, corners }) => ({
                bounds,
                corners,
                targets: [...iterators_1.reversed(findTargets_1.default(imageData, bounds))],
            })),
        ]);
        return {
            ballotImage: { imageData, metadata },
            contests,
        };
    }
    findContests(imageData) {
        // Try three columns, i.e. candidate pages.
        const shapesWithThreeColumns = [
            ...findContests_1.default(imageData, {
                columns: [true, true, true],
            }),
        ];
        if (shapesWithThreeColumns.length > 0) {
            return { contests: shapesWithThreeColumns, columns: 3 };
        }
        // Try two columns, i.e. measure pages.
        return {
            contests: [
                ...findContests_1.default(imageData, {
                    columns: [true, true],
                }),
            ],
            columns: 2,
        };
    }
    /**
     * Determines whether enough of the template images have been added to allow
     * scanning a ballot with the given metadata.
     */
    canScanBallot(metadata) {
        debug('canScanBallot metadata=%O', metadata);
        for (let pageNumber = 1; pageNumber <= metadata.pageNumber; pageNumber += 1) {
            const pageTemplate = this.getTemplate(Object.assign(Object.assign({}, metadata), { pageNumber }));
            if (!pageTemplate) {
                debug('cannot scan ballot because template page %d is missing', pageNumber);
                return false;
            }
            if (pageNumber === metadata.pageNumber &&
                pageTemplate.ballotImage.metadata.isTestMode !== metadata.isTestMode) {
                debug('cannot scan ballot because template page %d does not match the expected test ballot value (%s)', pageNumber, metadata.isTestMode);
                return false;
            }
        }
        return true;
    }
    /**
     * Interprets an image as a ballot, returning information about the votes cast.
     */
    async interpretBallot(imageData, metadata, { flipped = false, markScoreVoteThreshold = this.markScoreVoteThreshold, } = {}) {
        debug('interpretBallot: looking in %d×%d image', imageData.width, imageData.height);
        ({ imageData, metadata } = await this.normalizeImageDataAndMetadata(imageData, metadata, {
            flipped,
        }));
        if (metadata.isTestMode !== this.testMode) {
            throw new Error(`interpreter configured with testMode=${this.testMode} cannot interpret ballots with isTestMode=${metadata.isTestMode}`);
        }
        debug('using metadata: %O', metadata);
        const marked = await this.findMarks(imageData, metadata);
        const ballot = this.interpretMarks(marked, { markScoreVoteThreshold });
        return Object.assign(Object.assign({}, marked), { ballot });
    }
    async findMarks(imageData, metadata) {
        debug('looking for marks in %d×%d image', imageData.width, imageData.height);
        if (!this.canScanBallot(metadata)) {
            throw new Error('Cannot scan ballot because not all required templates have been added');
        }
        const { contests } = this.findContests(imageData);
        const ballotLayout = {
            ballotImage: { imageData, metadata },
            contests: [
                ...iterators_1.map(contests, ({ bounds, corners }) => ({
                    bounds,
                    corners,
                    options: [],
                })),
            ],
        };
        debug('found contest areas: %O', ballotLayout.contests.map(({ bounds }) => bounds));
        const { locales, ballotStyleId, precinctId, pageNumber } = metadata;
        const matchedTemplate = defined_1.default(this.getTemplate({ locales, ballotStyleId, precinctId, pageNumber }));
        const [mappedBallot, marks] = this.getMarksForBallot(ballotLayout, matchedTemplate, this.getContestsForTemplate(matchedTemplate));
        return { matchedTemplate, mappedBallot, metadata, marks };
    }
    /**
     * Get the contests for the given template.
     */
    getContestsForTemplate(template) {
        const { election } = this;
        const { locales, ballotStyleId, pageNumber, precinctId, } = template.ballotImage.metadata;
        const ballotStyle = defined_1.default(ballot_encoder_1.getBallotStyle({
            ballotStyleId,
            election,
        }));
        let contestOffset = 0;
        for (let i = 1; i < pageNumber; i += 1) {
            const pageTemplate = defined_1.default(this.getTemplate({ locales, ballotStyleId, precinctId, pageNumber: i }));
            contestOffset += pageTemplate.contests.length;
        }
        return ballot_encoder_1.getContests({ ballotStyle, election }).slice(contestOffset, contestOffset + template.contests.length);
    }
    interpretMarks({ marks, metadata }, { markScoreVoteThreshold = this.markScoreVoteThreshold, }) {
        const { election } = this;
        const ballotStyle = defined_1.default(ballot_encoder_1.getBallotStyle({
            ballotStyleId: metadata.ballotStyleId,
            election,
        }));
        const precinct = defined_1.default(ballot_encoder_1.getPrecinctById({
            election,
            precinctId: metadata.precinctId,
        }));
        return {
            ballotId: uuid_1.v4(),
            ballotStyle,
            ballotType: ballot_encoder_1.BallotType.Standard,
            isTestMode: metadata.isTestMode,
            precinct,
            votes: getVotesFromMarks_1.default(marks, { markScoreVoteThreshold }),
        };
    }
    async normalizeImageDataAndMetadata(imageData, metadata, { flipped = false } = {}) {
        binarize_1.binarize(imageData);
        if (metadata) {
            if (flipped) {
                flip_1.vh(imageData);
            }
        }
        else {
            const detectResult = await metadata_1.detect(this.election, imageData, {
                detectQRCode: this.detectQRCode,
            });
            metadata = detectResult.metadata;
            if (detectResult.flipped) {
                debug('detected image is flipped, correcting orientation');
                flip_1.vh(imageData);
            }
        }
        return { imageData, metadata };
    }
    getMarksForBallot(ballotLayout, template, contests) {
        assert_1.strict.equal(template.contests.length, contests.length, `template and election definition have different numbers of contests (${template.contests.length} vs ${contests.length}); maybe the template is from an old version of the election definition?`);
        assert_1.strict.equal(ballotLayout.contests.length, contests.length, `ballot and election definition have different numbers of contests (${ballotLayout.contests.length} vs ${contests.length}); maybe the ballot is from an old version of the election definition?`);
        const correspondance = findContests_1.findBallotLayoutCorrespondance(contests, ballotLayout, template);
        assert_1.strict(correspondance.corresponds, `ballot and template contest shapes do not correspond: ${util_1.inspect(correspondance, undefined, null)}`);
        const mappedBallot = this.mapBallotOntoTemplate(ballotLayout, template, {
            leftSideOnly: false,
        });
        const marks = [];
        const addCandidateMark = (contest, layout, option) => {
            const { score, offset } = this.targetMarkScore(template.ballotImage.imageData, mappedBallot, layout.target);
            debug(`'${option.id}' mark score: %d`, score);
            const mark = {
                type: 'candidate',
                bounds: layout.target.bounds,
                contest,
                option,
                score,
                scoredOffset: offset,
                target: layout.target,
            };
            marks.push(mark);
        };
        const addYesNoMark = (contest, layout, option) => {
            const { score, offset } = this.targetMarkScore(template.ballotImage.imageData, mappedBallot, layout.target);
            debug(`'${option}' mark score: %d`, score);
            const mark = {
                type: 'yesno',
                bounds: layout.target.bounds,
                contest,
                option,
                score,
                scoredOffset: offset,
                target: layout.target,
            };
            marks.push(mark);
        };
        const addEitherNeitherMark = (contest, layout, option) => {
            const { score, offset } = this.targetMarkScore(template.ballotImage.imageData, mappedBallot, layout.target);
            debug(`'${option === contest.eitherOption
                ? 'either'
                : option === contest.neitherOption
                    ? 'neither'
                    : option === contest.firstOption
                        ? 'first'
                        : 'second'}' mark score: %d`, score);
            const mark = {
                type: 'ms-either-neither',
                bounds: layout.target.bounds,
                contest,
                option,
                score,
                scoredOffset: offset,
                target: layout.target,
            };
            marks.push(mark);
        };
        for (const [{ options }, contest] of iterators_1.zip(template.contests, contests)) {
            debug(`getting marks for %s contest '%s'`, contest.type, contest.id);
            if (contest.type === 'candidate') {
                const expectedOptions = contest.candidates.length +
                    (contest.allowWriteIns ? contest.seats : 0);
                if (options.length !== expectedOptions) {
                    throw new Error(`Contest ${contest.id} is supposed to have ${expectedOptions} options(s), but found ${options.length}.`);
                }
                for (const [layout, candidate] of iterators_1.zipMin(options, contest.candidates)) {
                    addCandidateMark(contest, layout, candidate);
                }
                if (contest.allowWriteIns) {
                    const writeInOptions = options.slice(contest.candidates.length);
                    for (const [index, layout] of writeInOptions.entries()) {
                        addCandidateMark(contest, layout, {
                            id: `__write-in-${index}`,
                            name: 'Write-In',
                            isWriteIn: true,
                        });
                    }
                }
            }
            else if (contest.type === 'ms-either-neither') {
                if (options.length !== 4) {
                    throw new Error(`Contest ${contest.id} is supposed to have four options (either/neither/first/second), but found ${options.length}.`);
                }
                const [eitherLayout, neitherLayout, firstLayout, secondLayout] = options;
                addEitherNeitherMark(contest, eitherLayout, contest.eitherOption);
                addEitherNeitherMark(contest, neitherLayout, contest.neitherOption);
                addEitherNeitherMark(contest, firstLayout, contest.firstOption);
                addEitherNeitherMark(contest, secondLayout, contest.secondOption);
            }
            else {
                if (options.length !== 2) {
                    throw new Error(`Contest ${contest.id} is supposed to have two options (yes/no), but found ${options.length}.`);
                }
                const [yesLayout, noLayout] = options;
                addYesNoMark(contest, yesLayout, 'yes');
                addYesNoMark(contest, noLayout, 'no');
            }
        }
        return [mappedBallot, marks];
    }
    targetMarkScore(template, ballot, target, { zeroScoreThreshold = 0, highScoreThreshold = 0.25, maximumCorrectionPixelsX = 2, maximumCorrectionPixelsY = 2, } = {}) {
        debug('computing target mark score for target at (x=%d, y=%d) with zero threshold %d and high threshold %d', target.inner.x, target.inner.y, zeroScoreThreshold, highScoreThreshold);
        const offsetAndScore = new Map();
        const templateTarget = outline_1.default(crop_1.default(template, target.bounds));
        const templateTargetInner = outline_1.default(crop_1.default(template, target.inner));
        binarize_1.binarize(templateTarget);
        binarize_1.binarize(templateTargetInner);
        const templatePixelCountAvailableToFill = diff_1.countPixels(templateTargetInner, {
            color: binarize_1.PIXEL_WHITE,
        });
        for (const { x, y } of offsets_1.default()) {
            const xOver = Math.abs(x) > maximumCorrectionPixelsX;
            const yOver = Math.abs(y) > maximumCorrectionPixelsY;
            if (xOver && yOver) {
                break;
            }
            if (xOver || yOver) {
                continue;
            }
            const offsetTargetInner = Object.assign(Object.assign({}, target.inner), { x: target.inner.x + x, y: target.inner.y + y });
            const diffImageInner = diff_1.default(templateTarget, ballot, Object.assign(Object.assign({}, target.inner), { x: target.inner.x - target.bounds.x, y: target.inner.y - target.bounds.y }), offsetTargetInner);
            const ballotTargetInnerNewBlackPixelCount = diff_1.countPixels(diffImageInner, {
                color: binarize_1.PIXEL_BLACK,
            });
            const score = ballotTargetInnerNewBlackPixelCount / templatePixelCountAvailableToFill;
            if (score >= highScoreThreshold) {
                return { offset: { x, y }, score };
            }
            if (score <= zeroScoreThreshold) {
                debug('returning early at (x=%d, y=%d) since score (%d) is ≤ zero threshold (%d)', x, y, score, zeroScoreThreshold);
                return { offset: { x, y }, score };
            }
            offsetAndScore.set({ x, y }, score);
        }
        const [[minOffset, minScore]] = [...offsetAndScore].sort(([, a], [, b]) => a - b);
        debug('using minimum score %d from offset (x=%d, y=%d)', minScore, minOffset.x, minOffset.y);
        return { offset: minOffset, score: minScore };
    }
    mapImageWithPoints(imageMat, mappedSize, fromPoints, toPoints) {
        const mappedImage = new jsfeat.matrix_t(mappedSize.width, mappedSize.height, jsfeat.U8C1_t);
        if (fromPoints.length === 0) {
            // Nothing to guide mapping, so all we actually want to do is resize.
            // Note also that jsfeat generates a blank image if we try to do a warp
            // perspective with a homography containing no points.
            jsfeat.imgproc.resample(imageMat, mappedImage, mappedSize.width, mappedSize.height);
        }
        else {
            const homography = new jsfeat.motion_model.homography2d();
            const transform = new jsfeat.matrix_t(3, 3, jsfeat.F32_t | jsfeat.C1_t);
            homography.run(toPoints, fromPoints, transform, toPoints.length);
            jsfeat.imgproc.warp_perspective(imageMat, mappedImage, transform, 255);
        }
        return matToImageData_1.default(mappedImage);
    }
    mapBallotOntoTemplate(ballot, template, { leftSideOnly }) {
        const ballotMat = readGrayscaleImage_1.default(ballot.ballotImage.imageData);
        const templateSize = {
            width: template.ballotImage.imageData.width,
            height: template.ballotImage.imageData.height,
        };
        const ballotPoints = [];
        const templatePoints = [];
        for (const [{ corners: ballotContestCorners }, { bounds: templateContestBounds },] of iterators_1.zip(ballot.contests, template.contests)) {
            const [ballotTopLeft, ballotTopRight, ballotBottomLeft, ballotBottomRight,] = ballotContestCorners;
            const [templateTopLeft, templateTopRight, templateBottomLeft, templateBottomRight,] = geometry_1.rectCorners(templateContestBounds);
            if (leftSideOnly) {
                ballotPoints.push(ballotTopLeft, ballotBottomLeft);
                templatePoints.push(templateTopLeft, templateBottomLeft);
            }
            else {
                ballotPoints.push(ballotTopLeft, ballotTopRight, ballotBottomLeft, ballotBottomRight);
                templatePoints.push(templateTopLeft, templateTopRight, templateBottomLeft, templateBottomRight);
            }
        }
        return this.mapImageWithPoints(ballotMat, templateSize, ballotPoints, templatePoints);
    }
}
exports.default = Interpreter;
