"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.run = exports.printHelp = exports.printCommandHelp = exports.parseOptions = exports.description = exports.name = void 0;
const chalk_1 = __importDefault(require("chalk"));
const path_1 = require("path");
const helpCommand = __importStar(require("./help"));
const interpretCommand = __importStar(require("./interpret"));
const layoutCommand = __importStar(require("./layout"));
exports.name = 'help';
exports.description = 'Show help about a command';
const commands = [helpCommand, interpretCommand, layoutCommand];
async function parseOptions({ commandArgs, executablePath, }) {
    return {
        $0: path_1.basename(executablePath),
        command: commandArgs[0],
    };
}
exports.parseOptions = parseOptions;
function printGlobalHelp(options, out) {
    out.write(`Usage: ${options.$0} COMMAND [ARGS]\n`);
    out.write(`\n`);
    out.write(chalk_1.default.bold(`Commands:\n`));
    out.write(`\n`);
    const commandNameSpace = Math.max(...commands.map(({ name }) => name.length)) + 3;
    for (const { name, description } of commands) {
        out.write(`  ${chalk_1.default.bold(name)}${' '.repeat(commandNameSpace - name.length)}${description}\n`);
    }
    out.write(`\n`);
}
function printCommandHelp(options, out) {
    out.write(`Usage: ${options.$0} help COMMAND\n`);
    out.write(`\n`);
    out.write(`Print usage information for COMMAND.\n`);
}
exports.printCommandHelp = printCommandHelp;
function printHelp(options, out) {
    switch (options.command) {
        case undefined:
            printGlobalHelp(options, out);
            break;
        case 'help':
            printCommandHelp(options, out);
            break;
        case 'interpret':
            interpretCommand.printHelp(options.$0, out);
            break;
        case 'layout':
            layoutCommand.printHelp(options.$0, out);
            break;
        default:
            throw new Error(`unknown command: ${options.command}`);
    }
}
exports.printHelp = printHelp;
async function run(options, _stdin, stdout) {
    printHelp(options, stdout);
    return 0;
}
exports.run = run;
