"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseOptions = exports.parseGlobalOptions = exports.commands = exports.OptionParseError = void 0;
const helpCommand = __importStar(require("./commands/help"));
const interpretCommand = __importStar(require("./commands/interpret"));
const layoutCommand = __importStar(require("./commands/layout"));
class OptionParseError extends Error {
}
exports.OptionParseError = OptionParseError;
exports.commands = [helpCommand, interpretCommand, layoutCommand];
function parseGlobalOptions(args) {
    let help = false;
    let i = 2;
    let done = false;
    for (; i < args.length && !done; i += 1) {
        const arg = args[i];
        switch (arg) {
            case '-h':
            case '--help':
                help = true;
                break;
            default:
                if (!arg.startsWith('-')) {
                    done = true;
                    i -= 1;
                }
                else {
                    throw new OptionParseError(`Unknown global option: ${arg}`);
                }
                break;
        }
    }
    return {
        nodePath: args[0],
        executablePath: args[1],
        help,
        command: args[i],
        commandArgs: args.slice(i + 1),
    };
}
exports.parseGlobalOptions = parseGlobalOptions;
async function parseOptions(args) {
    const globalOptions = parseGlobalOptions(args);
    if (globalOptions.help) {
        return {
            command: helpCommand.name,
            options: await helpCommand.parseOptions(globalOptions),
        };
    }
    const { command } = globalOptions;
    switch (command) {
        case 'help':
            return {
                command,
                options: await helpCommand.parseOptions(globalOptions),
            };
        case 'interpret':
            return {
                command,
                options: await interpretCommand.parseOptions(globalOptions),
            };
        case 'layout':
            return {
                command,
                options: await layoutCommand.parseOptions(globalOptions),
            };
        default:
            throw new OptionParseError(`Unknown command: ${command}`);
    }
}
exports.parseOptions = parseOptions;
async function main(args, stdin, stdout, stderr) {
    try {
        const commandOptions = await parseOptions(args);
        switch (commandOptions.command) {
            case 'help':
                return await helpCommand.run(commandOptions.options, stdin, stdout);
            case 'interpret':
                return await interpretCommand.run(commandOptions.options, stdin, stdout);
            case 'layout':
                return await layoutCommand.run(commandOptions.options, stdin, stdout);
        }
    }
    catch (error) {
        if (error instanceof OptionParseError) {
            stderr.write(`error: ${error.message}\n`);
            const options = await helpCommand.parseOptions({
                nodePath: args[0],
                executablePath: args[1],
                help: true,
                command: 'help',
                commandArgs: [],
            });
            helpCommand.printHelp(options, stderr);
            return -1;
        }
        else {
            throw error;
        }
    }
}
exports.default = main;
