"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = __importDefault(require("debug"));
const votes_1 = require("./hmpb/votes");
const debug = debug_1.default('hmpb-interpreter:getVotesFromMarks');
/**
 * Gets the votes where the given marks have a high enough score to count, where
 * a score of 0 means nothing is filled in and a score of 1 means everything is
 * filled in.
 */
function getVotesFromMarks(marks, { markScoreVoteThreshold }) {
    const votes = {};
    for (const mark of marks) {
        switch (mark.type) {
            case 'candidate':
                if (mark.score >= markScoreVoteThreshold) {
                    debug(`'%s' contest '%s' mark score (%d) for '%s' meets vote threshold (%d)`, mark.type, mark.contest.id, mark.score, mark.option.name, markScoreVoteThreshold);
                    votes_1.addVote(votes, mark.contest, mark.option);
                }
                break;
            case 'yesno':
                if (mark.score >= markScoreVoteThreshold) {
                    debug(`'%s' contest '%s' mark score (%d) for '%s' meets vote threshold (%d)`, mark.type, mark.contest.id, mark.score, mark.option, markScoreVoteThreshold);
                    votes_1.addVote(votes, mark.contest, mark.option);
                }
                break;
            case 'ms-either-neither':
                if (mark.score >= markScoreVoteThreshold) {
                    debug(`'%s' contest '%s' mark score (%d) for '%s' meets vote threshold (%d)`, mark.type, mark.contest.id, mark.score, mark.option.label, markScoreVoteThreshold);
                    votes_1.addVote(votes, mark.contest, mark.option);
                }
                break;
            case 'stray':
                // TODO
                break;
            default:
                // @ts-expect-error - `mark` is `never` because the types say there are no other options
                throw new Error(`mark type '${mark.type}' is not yet supported`);
        }
    }
    return votes;
}
exports.default = getVotesFromMarks;
