"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Finds contest choice areas based on the contest box bounds and the contest
 * option target bounds.
 */
function findContestOptions(contests, { topMarginPercent = 3 } = {}) {
    return contests.map(({ bounds, corners, targets }) => targets.length === 1
        ? {
            bounds,
            corners,
            options: [{ bounds, target: targets[0] }],
        }
        : {
            bounds,
            corners,
            options: targets.map((target, i) => {
                const nextTarget = targets[i + 1];
                const height = nextTarget
                    ? Math.abs(nextTarget.bounds.y - target.bounds.y)
                    : Math.abs(bounds.y + bounds.height - target.bounds.y);
                const topMargin = Math.round((height * topMarginPercent) / 100);
                return {
                    bounds: {
                        x: bounds.x,
                        y: target.bounds.y - topMargin,
                        width: bounds.width,
                        height,
                    },
                    target,
                };
            }),
        });
}
exports.default = findContestOptions;
