"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = __importDefault(require("debug"));
const binarize_1 = require("../utils/binarize");
const geometry_1 = require("../utils/geometry");
const VisitedPoints_1 = require("../utils/VisitedPoints");
const shapes_1 = require("./shapes");
const debug = debug_1.default('hmpb-interpreter:findTargets');
function* findTargets(ballotImage, bounds, { inset = Math.round(0.0175 * ballotImage.width), aspectRatio = 1.5, aspectRatioTolerance = 0.1, expectedWidth = Math.round(0.025 * ballotImage.width), errorMargin = Math.ceil(0.04 * expectedWidth), } = {}) {
    debug('finding targets in %o', bounds);
    const visitedPoints = new VisitedPoints_1.VisitedPoints(ballotImage.width, ballotImage.height);
    const minAspectRatio = aspectRatio - aspectRatioTolerance;
    const maxAspectRatio = aspectRatio + aspectRatioTolerance;
    const x = bounds.x + Math.round(inset + expectedWidth / 2);
    let lastShape;
    for (let y = bounds.y + bounds.height - inset; y > bounds.y; y--) {
        const shape = shapes_1.findShape(ballotImage, { x, y }, visitedPoints);
        if (shape.bounds.width === 0 || shape.bounds.height === 0) {
            continue;
        }
        // If we already found one, so let's use it to determine the right size.
        let found;
        if (lastShape) {
            if (shape.bounds.width <= lastShape.bounds.width + errorMargin &&
                shape.bounds.width >= lastShape.bounds.width - errorMargin &&
                shape.bounds.height <= lastShape.bounds.height + errorMargin &&
                shape.bounds.height >= lastShape.bounds.height - errorMargin) {
                debug('shape matches last target shape!');
                found = true;
            }
            else {
                debug('skipping shape because it does not match the last known target shape: %O ≉ %O', shape.bounds, lastShape.bounds);
                found = false;
            }
        }
        else {
            const actualAspectRatio = shape.bounds.width / shape.bounds.height;
            if (actualAspectRatio < minAspectRatio ||
                actualAspectRatio > maxAspectRatio) {
                debug('skipping shape because it is the wrong aspect ratio: %d ≉ %d ± %d: %O', actualAspectRatio, aspectRatio, aspectRatioTolerance, shape.bounds);
                found = false;
            }
            else if (shape.bounds.width < expectedWidth - errorMargin ||
                shape.bounds.width > expectedWidth + errorMargin) {
                debug('skipping shape because it is the wrong width: %d ≉ %d ± %d: %O', shape.bounds.width, expectedWidth, errorMargin, shape.bounds);
                found = false;
            }
            else {
                found = true;
            }
        }
        if (found) {
            debug('found shape: %O', shape.bounds);
            const innerShape = shapes_1.findShape(ballotImage, geometry_1.rectCenter(shape.bounds, { round: true }), undefined, { color: binarize_1.PIXEL_WHITE });
            lastShape = shape;
            yield {
                bounds: shape.bounds,
                inner: innerShape.bounds,
            };
        }
        y = shape.bounds.y;
    }
}
exports.default = findTargets;
