"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.detect = exports.fromBytes = exports.fromString = exports.decodeSearchParams = exports.MetadataDecodeError = void 0;
const ballot_encoder_1 = require("@votingworks/ballot-encoder");
const defined_1 = __importDefault(require("./utils/defined"));
const qrcode = __importStar(require("./utils/qrcode"));
class MetadataDecodeError extends Error {
}
exports.MetadataDecodeError = MetadataDecodeError;
function decodeSearchParams(searchParams) {
    var _a, _b;
    const type = defined_1.default(searchParams.get('t'));
    const precinctId = defined_1.default(searchParams.get('pr'));
    const ballotStyleId = defined_1.default(searchParams.get('bs'));
    const pageInfo = defined_1.default(searchParams.get('p'));
    const primaryLocaleCode = (_a = searchParams.get('l1')) !== null && _a !== void 0 ? _a : undefined;
    const secondaryLocaleCode = (_b = searchParams.get('l2')) !== null && _b !== void 0 ? _b : undefined;
    let locales;
    if (primaryLocaleCode) {
        if (secondaryLocaleCode) {
            locales = { primary: primaryLocaleCode, secondary: secondaryLocaleCode };
        }
        else {
            locales = { primary: primaryLocaleCode };
        }
    }
    else {
        locales = { primary: 'en-US' };
    }
    const [typeTestMode] = type.split('', 2);
    const [pageInfoNumber] = pageInfo.split('-', 2);
    const isTestMode = typeTestMode === 't';
    const pageNumber = parseInt(pageInfoNumber, 10);
    return {
        electionHash: '',
        ballotType: ballot_encoder_1.BallotType.Standard,
        locales,
        ballotStyleId,
        precinctId,
        isTestMode,
        pageNumber,
    };
}
exports.decodeSearchParams = decodeSearchParams;
function isBase64(string) {
    return Buffer.from(string, 'base64').toString('base64') === string;
}
function fromString(election, text) {
    if (isBase64(text)) {
        return fromBytes(election, Buffer.from(text, 'base64'));
    }
    return decodeSearchParams(new URL(text).searchParams);
}
exports.fromString = fromString;
function fromBytes(election, data) {
    if (data[0] === 'V'.charCodeAt(0) && data[1] === 'P'.charCodeAt(0)) {
        return ballot_encoder_1.v1.decodeHMPBBallotPageMetadata(election, data);
    }
    return fromString(election, new TextDecoder().decode(data));
}
exports.fromBytes = fromBytes;
async function detect(election, imageData, { detectQRCode = qrcode.default } = {}) {
    const result = await detectQRCode(imageData);
    if (!result) {
        throw new MetadataDecodeError('Expected QR code not found.');
    }
    return {
        metadata: fromBytes(election, result.data),
        flipped: result.rightSideUp === false,
    };
}
exports.detect = detect;
