"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.binarize = exports.RGBA_WHITE = exports.RGBA_BLACK = exports.PIXEL_WHITE = exports.PIXEL_BLACK = void 0;
const grayscale_1 = __importDefault(require("./grayscale"));
const imageFormatUtils_1 = require("./imageFormatUtils");
const otsu_1 = __importDefault(require("./otsu"));
exports.PIXEL_BLACK = 0;
exports.PIXEL_WHITE = (1 << 8) - 1;
exports.RGBA_BLACK = [exports.PIXEL_BLACK, exports.PIXEL_BLACK, exports.PIXEL_BLACK, 0xff];
exports.RGBA_WHITE = [exports.PIXEL_WHITE, exports.PIXEL_WHITE, exports.PIXEL_WHITE, 0xff];
/**
 * Converts an image to a grayscale image with all pixels set to either black or
 * white, depending on whether their luminosity is higher or lower than
 * `threshold`. By default an automatic threshold is calculated using Otsu's
 * method.
 *
 * Operates on an image in-place by default, or you may specify a different
 * destination image.
 */
function binarize(srcImageData, dstImageData = srcImageData, { threshold } = {}) {
    grayscale_1.default(srcImageData, dstImageData);
    const { data: dst } = dstImageData;
    threshold !== null && threshold !== void 0 ? threshold : (threshold = otsu_1.default(dstImageData.data, imageFormatUtils_1.getImageChannelCount(dstImageData)));
    if (imageFormatUtils_1.isRGBA(dstImageData)) {
        const dst32 = new Int32Array(dst.buffer);
        const RGBA_WHITE = exports.PIXEL_WHITE | (exports.PIXEL_WHITE << 8) | (exports.PIXEL_WHITE << 16) | (0xff << 24);
        const RGBA_BLACK = exports.PIXEL_BLACK | (exports.PIXEL_BLACK << 8) | (exports.PIXEL_BLACK << 16) | (0xff << 24);
        for (let offset32 = 0, offset8 = 0, size = dst32.length; offset32 < size; offset32++, offset8 += 4) {
            dst32[offset32] = dst[offset8] < threshold ? RGBA_BLACK : RGBA_WHITE;
        }
    }
    else {
        for (let offset = 0, size = dst.length; offset < size; offset++) {
            dst[offset] = dst[offset] < threshold ? exports.PIXEL_BLACK : exports.PIXEL_WHITE;
        }
    }
}
exports.binarize = binarize;
