"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isRGBA = exports.isGrayscale = exports.getImageChannelCount = exports.assertSizesMatch = exports.assertImageChannelsMatch = exports.assertRGBAOrGrayscaleImage = exports.assertGrayscaleImage = exports.assertRGBAImage = exports.makeImageTransform = exports.makeInPlaceImageTransform = void 0;
const assert_1 = require("assert");
function makeInPlaceImageTransform(gray, rgba) {
    return (srcImageData, dstImageData, ...args) => {
        const channels = srcImageData.data.length / (srcImageData.width * srcImageData.height);
        switch (channels) {
            case 1:
                return gray(srcImageData, dstImageData, ...args);
            case 4:
                return rgba(srcImageData, dstImageData, ...args);
            default:
                throw new Error(`unexpected ${channels}-channel image`);
        }
    };
}
exports.makeInPlaceImageTransform = makeInPlaceImageTransform;
function makeImageTransform(gray, rgba) {
    return (imageData, ...args) => {
        const channels = imageData.data.length / (imageData.width * imageData.height);
        switch (channels) {
            case 1:
                return gray(imageData, ...args);
            case 4:
                return rgba(imageData, ...args);
            default:
                throw new Error(`unexpected ${channels}-channel image`);
        }
    };
}
exports.makeImageTransform = makeImageTransform;
function assertRGBAImage(imageData) {
    assert_1.strict.equal(imageData.data.length, imageData.width * imageData.height * 4, 'expected 4-channel RGBA image');
}
exports.assertRGBAImage = assertRGBAImage;
function assertGrayscaleImage(imageData) {
    assert_1.strict.equal(imageData.data.length, imageData.width * imageData.height, 'expected 1-channel grayscale image');
}
exports.assertGrayscaleImage = assertGrayscaleImage;
function assertRGBAOrGrayscaleImage(imageData) {
    if (!isRGBA(imageData) && !isGrayscale(imageData)) {
        assert_1.strict.fail('expected 1-channel grayscale or 4-channel RGBA image');
    }
}
exports.assertRGBAOrGrayscaleImage = assertRGBAOrGrayscaleImage;
function assertImageChannelsMatch(imageData1, imageData2) {
    assert_1.strict.equal(getImageChannelCount(imageData1), getImageChannelCount(imageData2), 'expected images to have the same number of channels');
}
exports.assertImageChannelsMatch = assertImageChannelsMatch;
function assertSizesMatch(size1, size2) {
    assert_1.strict.deepEqual({ width: size1.width, height: size1.height }, { width: size2.width, height: size2.height }, 'expected sizes to be equal');
}
exports.assertSizesMatch = assertSizesMatch;
function getImageChannelCount(imageData) {
    return imageData.data.length / (imageData.width * imageData.height);
}
exports.getImageChannelCount = getImageChannelCount;
function isGrayscale(imageData) {
    return getImageChannelCount(imageData) === 1;
}
exports.isGrayscale = isGrayscale;
function isRGBA(imageData) {
    return getImageChannelCount(imageData) === 4;
}
exports.isRGBA = isRGBA;
