"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.map = exports.reversed = exports.zipMin = exports.zip = void 0;
const util_1 = require("util");
function* zip(...iterables) {
    const iterators = iterables.map((iterable) => iterable[Symbol.iterator]());
    while (true) {
        const nexts = iterators.map((iterator) => iterator.next());
        const dones = nexts.filter(({ done }) => done);
        if (dones.length === nexts.length) {
            break;
        }
        else if (dones.length > 0) {
            throw new Error(`not all iterables are the same length: ${nexts
                .map(({ value }) => util_1.inspect(value))
                .join(', ')}`);
        }
        yield nexts.map(({ value }) => value);
    }
}
exports.zip = zip;
function* zipMin(left, right) {
    const leftIterator = left[Symbol.iterator]();
    const rightIterator = right[Symbol.iterator]();
    while (true) {
        const leftNext = leftIterator.next();
        const rightNext = rightIterator.next();
        if (leftNext.done || rightNext.done) {
            break;
        }
        yield [leftNext.value, rightNext.value];
    }
}
exports.zipMin = zipMin;
function* reversed(collection) {
    const elements = [...collection];
    for (let i = elements.length - 1; i >= 0; i -= 1) {
        yield elements[i];
    }
}
exports.reversed = reversed;
function* map(collection, mapfn) {
    let index = 0;
    for (const element of collection) {
        yield mapfn(element, index);
        index += 1;
    }
}
exports.map = map;
