"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const binarize_1 = require("./binarize");
const canvas_1 = require("./canvas");
const imageFormatUtils_1 = require("./imageFormatUtils");
/**
 * Outline pixels of a certain color with the same color.
 */
function outline({ data: src, width, height }, { color = binarize_1.PIXEL_BLACK } = {}) {
    const channels = imageFormatUtils_1.getImageChannelCount({ data: src, width, height });
    const result = canvas_1.createImageData(width, height);
    const v1px = width * channels;
    const h1px = channels;
    const pixel = channels === 4 ? [color, color, color, 0xff] : [color];
    const { data: dst } = result;
    dst.set(src);
    for (let y = 0; y < height; y += 1) {
        for (let x = 0; x < width; x += 1) {
            const offset = (y * width + x) * channels;
            if (src[offset] === color) {
                if (y > 0) {
                    dst.set(pixel, offset - v1px);
                }
                if (y < height - 1) {
                    dst.set(pixel, offset + v1px);
                }
                if (x > 0) {
                    dst.set(pixel, offset - h1px);
                }
                if (x < width - 1) {
                    dst.set(pixel, offset + h1px);
                }
            }
        }
    }
    return result;
}
exports.default = outline;
