"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.detect = void 0;
const jsqr_1 = __importDefault(require("jsqr"));
const withCropping_1 = require("./withCropping");
const debug_1 = __importDefault(require("debug"));
const debug = debug_1.default('hmpb-interpreter:jsqr');
/**
 * Uses jsQR to detect QR codes in a ballot image.
 */
async function detect(imageData) {
    debug('detecting QR code in %d×%d image', imageData.width, imageData.height);
    const { data, width, height } = imageData;
    const qrcode = jsqr_1.default(data, width, height);
    if (!qrcode) {
        return;
    }
    const qrdata = Buffer.from(qrcode.binaryData);
    const rightSideUp = qrcode.location.bottomRightCorner.x > qrcode.location.topLeftCorner.x;
    return {
        data: qrdata,
        rightSideUp,
    };
}
exports.detect = detect;
exports.default = withCropping_1.withCropping(detect);
