"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.withCropping = void 0;
const crop_1 = __importDefault(require("../crop"));
const geometry_1 = require("../geometry");
/**
 * Makes a new QR code detector from an existing one that works by cropping the
 * bottom-right and top-left corners of the image before passing it on.
 */
function withCropping(decode, { widthFraction = 1 / 4, heightFraction = 1 / 5 } = {}) {
    return async (imageData) => {
        var _a, _b;
        const width = Math.floor(imageData.width * widthFraction);
        const height = Math.floor(imageData.height * heightFraction);
        const searchBounds = {
            x: imageData.width - width,
            y: imageData.height - height,
            width,
            height,
        };
        {
            const cropped = crop_1.default(imageData, searchBounds);
            const decoded = await decode(cropped);
            if (decoded) {
                return {
                    data: decoded.data,
                    rightSideUp: (_a = decoded.rightSideUp) !== null && _a !== void 0 ? _a : true,
                };
            }
        }
        {
            const cropped = crop_1.default(imageData, geometry_1.flipRectVH({ x: 0, y: 0, width: imageData.width, height: imageData.height }, searchBounds));
            const decoded = await decode(cropped);
            if (decoded) {
                return {
                    data: decoded.data,
                    rightSideUp: (_b = decoded.rightSideUp) !== null && _b !== void 0 ? _b : false,
                };
            }
        }
        return undefined;
    };
}
exports.withCropping = withCropping;
